#include <LiquidCrystal_I2C.h> //We add the libraries for the LCD screen with your i2c adapter
#include <Wire.h>

#define pinBuzzer 12 //Buzzer pin
#define NUMCAL 9 //number of pumpkins
int led[NUMCAL] = {32, 34, 36, 38, 40, 42, 44, 46, 48}; // LEDs pins
int botton[NUMCAL] = {33, 35, 37, 39, 41, 43, 45, 47, 49}; //Buttons pins
int lastState[NUMCAL] = {HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH}; //Last state of the buttons

#define bottonStart 24 // Start Button pin
#define Buzzer 51 //Buzzer pin

int score = 0; //start variable score
int timer_max = 30000; //time of the game
int record = 0; //start variable record

int currentLed; //actual high led
int lastRoundButton = -1; //variable used as a memory of the last button pressed

bool gameStarted = false; //state of the game
unsigned long t; //moment of time when you start the game
bool play = false; //state of the`play

LiquidCrystal_I2C lcd(0x27, 16, 2); //definition of LCD

void setup() {
  Serial.begin(9600);    //start the serial
  for (int i = 0; NUMCAL > i; i++)  //define all the elements of the button array as output
  {
  {
    pinMode(led[i], OUTPUT); 
  }

  for (int j = 0; NUMCAL > j; j++) //define all the elements of the button array as input pullup
  {
    pinMode(botton[j], INPUT_PULLUP);
  }

  pinMode(bottonStart , INPUT_PULLUP); //definition of the buttonStart as an input pullup
  
  lcd.init(); // initialize the LCD
  lcd.backlight(); //turn on the backlight of the LCD.

  lcd.print("UATEA HALLOWEEN"); 
  lcd.setCursor(0, 1); // place the cursor in the first position of the second line (column:0; line:1) 
  lcd.print("ANNA, MAITA y MJ"); 
}

void loop() {
  if (digitalRead(bottonStart) == LOW && !gameStarted) // condition to know when to start the game
  {
    gameStarted = true; // game state variable initialization 
    lcd.clear(); // clean the information of the LCD screen
    lcd.print("START!");
    lcd.setCursor(0, 1);
    lcd.print("Last record ");
    lcd.print(record); //show the last record 
    delay(2000); // time between button start pressed and when the game starts
    t = millis(); //definition of the time that starts the game (start button pressed+2s)
    currentLed = newLed(-1);
    lcd.clear();
  }

  if (gameStarted && millis() - t < timer_max) //condition to know if the game has started and the time is less than 30s (timer_max)
  {
    int times = (30000 - (millis() - t)) / 1000; //countdown from timer_max to 0s
    lcd.setCursor(0, 0);
    lcd.print("Tiempo ");
    if (times < 10) //correction of the countdown when there is only one digit 
    {
      lcd.setCursor(8, 0);
      lcd.print(" ");
      lcd.setCursor(7, 0);
    }
    lcd.print(times);
    lcd.setCursor(0, 1); 
    lcd.print("Score "); 
    lcd.print(score); //show the actualized score

    digitalWrite(led[currentLed], HIGH); //turn on a random LED

    int buttonRead = readButtons(); //call the function readButton to create a new random number
    if (buttonRead >= 0) //to control when the first random LED is turned on.
    {
      if (buttonRead == currentLed) //condition to know if you press the correct button
      {
        digitalWrite(led[currentLed], LOW); //turn off the LED
        score++; // increase the score (+1)
        lastRoundButton = currentLed; // update the variable lastRoundButton
        currentLed = newLed(currentLed); // create a new random number
      }
      else if (buttonRead != currentLed && buttonRead != lastRoundButton) //condition to know if a wrong button has been pressed
      {
        digitalWrite(led[currentLed], LOW); //turn off the led
        score--; // decrease the score (-1)
        tone(pinBuzzer, 200); //to make the error sound with the buzzer 
        delay(500); //time that the sound is on
        noTone(pinBuzzer); //stop the sound
      }
    }
    delay(100); //used to improve LCD performance
  }
  else if (gameStarted && millis() - t > timer_max) //condition to know if the play has finished 
  {
    for (int h = 0; NUMCAL > h; h++)
    {
      digitalWrite(led[h], LOW); //turn off all the LEDs 
    }

    lcd.clear();
    if (score > record) //condition to know if the user has made a record
    {
      record = score; //update the variable record with the newest score 
      lcd.print("NEW RECORD!");
      lcd.setCursor(0, 1); 
      lcd.print(score); //print final score in the LCD
    }

    else //no new record
    {
      lcd.print("Score");
      lcd.setCursor(0, 1); 
      lcd.print(score); 
    }
    delay(5000);
    lcd.clear();
    lcd.print("Press start to"); //message that indicates how to start a new play
    lcd.setCursor(0, 1); 
    lcd.print("play again");
    gameStarted = false; //finish the play
    score = 0; //update the score (restart the variable)
  }
}

int newLed(int current) //function to create random number without repeating the last one
{
  int newLed = random(0, NUMCAL);
  while (current == newLed)
  {
    newLed = random(0, NUMCAL);
  }
  return newLed;
}

int readButtons () //function to know which button has been pressed
{
  int newReading;
  for (int i = 0; i < NUMCAL; i++)
  {
    newReading = digitalRead(botton[i]);
    if (lastState[i] == HIGH && newReading == LOW)
    {
      lastState[i] = newReading;
      Serial.print("Button pressed ");
      Serial.println(i);
      return i;
    }
    lastState[i] = newReading;
  }
  return -1;
}
